@@ -19,38 +19,33 @@ Suppressed = {}
 
 for s in [
         ' exited with ',  # Is hit; driver.py filters out
+        ' loading non-variable',  # Instead 'storing to parameter' or syntax error
+        '--pipe-filter: Can\'t pipe: ',  # Can't test
+        '--pipe-filter: fork failed: ',  # Can't test
+        'Assigned pin is neither input nor output',  # Instead earlier error
+        'Define missing argument \'',  # Instead get Define passed too many arguments
+        'Define or directive not defined: `',  # Instead V3ParseImp will warn
         'EOF in unterminated string',  # Instead get normal unterminated
         'Enum ranges must be integral, per spec',  # Hard to hit
+        'Expecting define formal arguments. Found: ',  # Instead define syntax error
         'Import package not found: ',  # Errors earlier, until future parser released
+        'Member selection of non-struct/union object \'',  # Instead dotted expression error or V3Link other
         'Return with return value isn\'t underneath a function',  # Hard to hit, get other bad return messages
-        'Syntax error: Range \':\', \'+:\' etc are not allowed in the instance ',  # Instead get syntax error
         'Syntax error parsing real: \'',  # Instead can't lex the number
+        'Syntax error: Range \':\', \'+:\' etc are not allowed in the instance ',  # Instead get syntax error
         'Unsupported: Ranges ignored in port-lists',  # Hard to hit
         'dynamic new() not expected in this context (expected under an assign)',  # Instead get syntax error
         # Not yet analyzed
-        ' loading non-variable',
-        '--pins-bv maximum is 65: ',
         '--pipe-filter protocol error, unexpected: ',
         '--pipe-filter returned bad status',
-        '--pipe-filter: Can\'t pipe: ',
-        '--pipe-filter: fork failed: ',
-        '--threads must be >= 0: ',
-        '--threads-max-mtasks must be >= 1: ',
-        '--trace-threads must be >= 1: ',
-        '/*verilator sformat*/ can only be applied to last argument of ',
         'Argument needed for string.',
         'Array initialization has too few elements, need element ',
-        'Assigned pin is neither input nor output',
         'Assignment pattern with no members',
         'Can\'t find varpin scope of ',
         'Can\'t read annotation file: ',
         'Can\'t resolve module reference: \'',
         'Can\'t write file: ',
-        'Circular logic when ordering code (non-cutable edge loop)',
-        'Define missing argument \'',
-        'Define or directive not defined: `',
-        'Exceeded limit of ',
-        'Expecting define formal arguments. Found: ',
+        'Expected data type, not a ',
         'Extern declaration\'s scope is not a defined class',
         'File not found: ',
         'Format to $display-like function must have constant format string',
@@ -58,31 +53,31 @@ for s in [
         'Illegal +: or -: select; type already selected, or bad dimension: ',
         'Illegal bit or array select; type already selected, or bad dimension: ',
         'Illegal range select; type already selected, or bad dimension: ',
-        'Member selection of non-struct/union object \'',
+        'Interface port ',
+        'Interface port declaration ',
         'Modport item is not a function/task: ',
         'Modport item is not a variable: ',
         'Modport item not found: ',
         'Modport not referenced as <interface>.',
         'Modport not referenced from underneath an interface: ',
+        'Non-interface used as an interface: ',
         'Parameter type pin value isn\'t a type: Param ',
         'Parameter type variable isn\'t a type: Param ',
         'Pattern replication value of 0 is not legal.',
+        'Reference to \'',
         'Signals inside functions/tasks cannot be marked forceable',
         'Slice size cannot be zero.',
         'Slices of arrays in assignments have different unpacked dimensions, ',
         'String of ',
         'Symbol matching ',
         'Unexpected connection to arrayed port',
-        'Unmatched brackets in variable substitution in file: ',
         'Unsized numbers/parameters not allowed in streams.',
         'Unsupported RHS tristate construct: ',
         'Unsupported or syntax error: Unsized range in instance or other declaration',
         'Unsupported pullup/down (weak driver) construct.',
         'Unsupported tristate construct (not in propagation graph): ',
         'Unsupported tristate port expression: ',
-        'Unsupported/unknown built-in dynamic array method ',
         'Unsupported: $bits for queue',
-        'Unsupported: $c can\'t generate wider than 64 bits',
         'Unsupported: &&& expression',
         'Unsupported: +%- range',
         'Unsupported: +/- range',
@@ -98,7 +93,6 @@ for s in [
         'Unsupported: Public functions with >64 bit outputs; ',
         'Unsupported: Replication to form ',
         'Unsupported: Shifting of by over 32-bit number isn\'t supported.',
-        'Unsupported: Signal strengths are unsupported ',
         'Unsupported: Size-changing cast on non-basic data type',
         'Unsupported: Slice of non-constant bounds',
         'Unsupported: Unclocked assertion',
@@ -107,27 +101,18 @@ for s in [
         'Unsupported: [] dimensions',
         'Unsupported: \'default :/\' constraint',
         'Unsupported: \'{} .* patterns',
-        'Unsupported: \'{} tagged patterns',
-        'Unsupported: always[] (in property expression)',
         'Unsupported: assertion items in clocking blocks',
-        'Unsupported: default clocking identifier',
         'Unsupported: don\'t know how to deal with ',
         'Unsupported: eventually[] (in property expression)',
         'Unsupported: extern forkjoin',
-        'Unsupported: extern interface',
-        'Unsupported: extern module',
         'Unsupported: extern task',
         'Unsupported: modport export',
         'Unsupported: no_inline for tasks',
         'Unsupported: property port \'local\'',
-        'Unsupported: randsequence production list',
-        'Unsupported: randsequence repeat',
         'Unsupported: repeat event control',
-        'Unsupported: s_always (in property expression)',
         'Unsupported: static cast to ',
         'Unsupported: super',
         'Unsupported: this.super',
-        'Unsupported: trireg',
         'Unsupported: with[] stream expression',
 ]:
     Suppressed[s] = True
@@ -196,7 +181,7 @@ def check():
     read_outputs()
 
     print("Number of suppressions = " + str(len(Suppressed)))
-    print("Coverage = %3.1f%%" % (100 - int(100 * len(Suppressed) / len(Messages))))
+    print("Coverage = %3.1f%%" % (100 - (100 * len(Suppressed) / len(Messages))))
     print()
 
     print("Checking for v3error/v3warn messages in sources without")
